package org.kodein.db.impl.model.cache

import org.kodein.db.*
import org.kodein.db.model.ModelBatch
import org.kodein.memory.Closeable
import org.kodein.memory.util.MaybeThrowable
import org.kodein.memory.util.forEachCatchTo
import kotlin.reflect.KClass

internal class CachedModelBatch(private val cmdb: CachedModelDB, private val batch: ModelBatch) : ModelBatch, KeyMaker by cmdb, ValueMaker by cmdb, Closeable by batch {

    private val reacts = ArrayList<CachedModelDB.() -> Unit>()

    override fun <M : Any> put(model: M, vararg options: Options.BatchPut): KeyAndSize<M> {
        val key = keyFrom(model)
        val size = batch.put(key, model, *options)
        reacts.add { didPut(model, key, size, options) }
        return KeyAndSize(key, size)
    }

    override fun <M : Any> put(key: Key<M>, model: M, vararg options: Options.BatchPut): Int {
        val size = batch.put(key, model, *options)
        reacts.add { didPut(model, key, size, options) }
        return size
    }

    override fun <M : Any> delete(type: KClass<M>, key: Key<M>, vararg options: Options.BatchDelete) {
        batch.delete(type, key, *options)
        reacts.add { didDelete(key, options) }
    }

    override fun write(afterErrors: MaybeThrowable, vararg options: Options.BatchWrite) {
        batch.write(afterErrors, *(options + ReactInLock {
            reacts.forEachCatchTo(afterErrors) { cmdb.it() }
        }))
    }

}
